@@ -3,7 +3,7 @@
 // The University of Illinois/NCSA
 // Open Source License (NCSA)
 //
-// Copyright (c) 2014-2020, Advanced Micro Devices, Inc. All rights reserved.
+// Copyright (c) 2014-2024, Advanced Micro Devices, Inc. All rights reserved.
 //
 // Developed by:
 //
@@ -60,9 +60,17 @@
 
 #include "core/util/utils.h"
 #include "core/util/locks.h"
+#include "core/util/timer.h"
 
 #include "inc/amd_hsa_signal.h"
 
+#if defined(__i386__) || defined(__x86_64__)
+#include <mwaitxintrin.h>
+#ifndef MWAITX_ECX_TIMER_ENABLE
+#define MWAITX_ECX_TIMER_ENABLE 0x2  // BIT(1)
+#endif
+#endif
+
 // Allow hsa_signal_t to be keys in STL structures.
 namespace std {
 template <> struct less<hsa_signal_t> {
@@ -76,6 +84,50 @@ template <> struct less<hsa_signal_t> {
 }
 
 namespace rocr {
+namespace timer {
+inline timer::fast_clock::duration GetFastTimeout(uint64_t timeout) {
+  uint64_t hsa_freq = 0;
+  HSA::hsa_system_get_info(HSA_SYSTEM_INFO_TIMESTAMP_FREQUENCY, &hsa_freq);
+  return timer::duration_from_seconds<timer::fast_clock::duration>(
+      double(timeout) / double(hsa_freq));
+}
+
+inline void CheckAbortTimeout(const timer::fast_clock::time_point& start_time,
+                            uint32_t signal_abort_timeout) {
+  if (signal_abort_timeout) {
+    const timer::fast_clock::duration abort_timeout =
+        std::chrono::seconds(signal_abort_timeout);
+    if (timer::fast_clock::now() - start_time > abort_timeout) {
+      throw AMD::hsa_exception(HSA_STATUS_ERROR_FATAL,
+                             "Signal wait abort timeout.\n");
+    }
+  }
+}
+
+inline void DoMwaitx(int64_t* addr, uint32_t timeout, bool timer_enable = false) {
+#if defined(__i386__) || defined(__x86_64__)
+  _mm_monitorx(addr, 0, 0);
+  _mm_mwaitx(0, timeout, timer_enable ? MWAITX_ECX_TIMER_ENABLE : 0);
+#endif
+}
+} // namespace timer
+
+inline bool CheckSignalCondition(int64_t value, hsa_signal_condition_t condition,
+                               hsa_signal_value_t compare_value) {
+  switch (condition) {
+    case HSA_SIGNAL_CONDITION_EQ:
+      return value == compare_value;
+    case HSA_SIGNAL_CONDITION_NE:
+      return value != compare_value;
+    case HSA_SIGNAL_CONDITION_GTE:
+      return value >= compare_value;
+    case HSA_SIGNAL_CONDITION_LT:
+      return value < compare_value;
+    default:
+      return false;
+  }
+}
+
 namespace core {
 class Agent;
 class Signal;
@@ -90,7 +142,11 @@ struct SharedSignal {
   uint64_t sdma_end_ts;
   uint8_t reserved2[24];
 
-  SharedSignal() {
+  SharedSignal() :
+    sdma_start_ts(0),
+    reserved{},
+    sdma_end_ts(0),
+    reserved2{} {
     memset(&amd_signal, 0, sizeof(amd_signal));
     amd_signal.kind = AMD_SIGNAL_KIND_INVALID;
     core_signal = nullptr;
@@ -347,14 +403,30 @@ class Signal {
   /// Returns NULL for DefaultEvent Type.
   virtual HsaEvent* EopEvent() = 0;
 
-  /// @brief Waits until any signal in the list satisfies its condition or
-  /// timeout is reached.
-  /// Returns the index of a satisfied signal.  Returns -1 on timeout and
-  /// errors.
-  static uint32_t WaitAny(uint32_t signal_count, const hsa_signal_t* hsa_signals,
-                          const hsa_signal_condition_t* conds, const hsa_signal_value_t* values,
-                          uint64_t timeout_hint, hsa_wait_state_t wait_hint,
-                          hsa_signal_value_t* satisfying_value);
+  /// @brief Waits until multiple signals in the list satisfy their conditions
+  /// or a timeout is reached.
+  /// @param signal_count Number of hsa_signals in the list.
+  /// @param hsa_signals Pointer to array of HSA signals.
+  /// @param conds Pointer to array of signal conditions.
+  /// @param values Pointer to array of signal values.
+  /// @param timeout Timeout hint value.
+  /// @param wait_hint Hint about wait state.
+  /// @param satisfying_values Vector of satisfying values. If \p wait_on_all
+  /// is false (then we are waiting on any signal in the list) this will contain
+  /// only the first satisfying value.
+  /// @param wait_on_all Wait on all signals in the list to satisfy their
+  /// conditions if true, else wait on any signal in the list to satisfy its
+  /// condition.
+  /// @return Return the index of the first signal in the list that satisfies
+  /// its condition or -1 on a timeout. Note that if \p wait_on_all is true,
+  /// then all signals in the list satisfy their conditions, thus the index will
+  /// always be 0.
+  static uint32_t WaitMultiple(uint32_t signal_count, const hsa_signal_t* hsa_signals,
+                               const hsa_signal_condition_t* conds,
+                               const hsa_signal_value_t* values, uint64_t timeout,
+                               hsa_wait_state_t wait_hint,
+                               std::vector<hsa_signal_value_t>& satisfying_values,
+                               bool wait_on_all);
 
   /// @brief Dedicated funtion to wait on signals that are not of type HSA_EVENTTYPE_SIGNAL
   /// these events can only be received by calling the underlying driver (i.e via the hsaKmtWaitOnMultipleEvents_Ext
